
#include "mcc_generated_files/mcc.h"

#define keyDASH     !RA4
#define keyDOT      !RC2
#define lenSp       lenDOT
#define CWout       LATA5

// Sin定数　20分割　MAX 99
uint8_t wave[] = {
     97,90,79,65,50,34,20,9,2,0,2,9,20,34,50,65,79,90,97,99
};

/* ++++++++  共通変数 ++++++++ */
uint8_t p_wave;
int  lenDOT = 100;
bool flagDASH = false;
bool flagDOT  = false;

/* ++++++++  プロトタイプ ++++++++ */
void SigDOT(void);
void delayDOT (int);
void SigDASH(void);
void delayDASH (int);
void changeFreq(void);
void MyTMR_ISR(void);      // 割り込み処理関数プロトタイプ

/*  Main application */
void main(void)
{
    SYSTEM_Initialize();
    
    ADCON0bits.CHS = 4;         // ADチャンネル設定
    __delay_ms(5);              // ADチャージ時間
    GO = 1;                     // AD変換開始
    while(GO);                  // AD変換終了待ち
    lenDOT = (ADRESH / 2) + 60; // AD変換値を端点時間にセット
    
    TMR2_SetInterruptHandler(MyTMR_ISR);    // 割り込み処理関数設定
    INTERRUPT_GlobalInterruptEnable();
    INTERRUPT_PeripheralInterruptEnable();
    
    while (1)
    {
        if(flagDOT || keyDOT) SigDOT();     // 短点が押されたなら端点処理
        if(flagDASH || keyDASH) SigDASH();  // 長点が押されたなら長点処理
        if(!RC4) changeFreq();              // SWが押れたら Side Tone変更
    }
}

//++++++++++++++++++++++++++++++++++++++++++++++
// 短点とマーク間スペースを送信
// この間、長点キーを繰り返しチェックし押されれば記憶する
// スペース開始前にVRのAD変換を開始し
// スペース終了後に変換値を送信速度に変換し保存
//++++++++++++++++++++++++++++++++++++++++++++++
void SigDOT(void){
    CWout = 1;                      // CW出力　On
    delayDOT(lenDOT);               // 文字間スペース開始
    CWout = 0;                      // CW出力　Off
    ADCON0bits.GO=1;                // AD変換開始
    delayDOT(lenSp);                // 文字間スペース開始
    lenDOT = (ADRESH / 2) + 60;     // AD変換値を端点時間にセット
    if(!keyDOT)flagDOT = false;     // 短点Keyが離れていれば、フラグクリア
}

void delayDOT(int len){
    while(len--){                       // 指定の長さだけ繰り返す
        __delay_ms(1);                  // 1ms 待ち
        if(keyDASH) flagDASH = true;    // 長点が押されたなら長点記憶
    }
}

    
//++++++++++++++++++++++++++++++++++++++++++++++
// 長点とマーク間スペースを送信
// この間、短点キーを繰り返しチェックし押されれば記憶する
// 長点時間は短点の３倍
//++++++++++++++++++++++++++++++++++++++++++++++
void SigDASH(void){
    CWout = 1;                      // CW出力　On
    delayDASH(lenDOT * 3);          // 長点の長さだけ遅延
    CWout = 0;                      // CW出力　Off
    ADCON0bits.GO=1;                // AD変換開始
    delayDASH(lenSp);                // 文字間の長さだけ遅延
    lenDOT = (ADRESH / 2) + 60;     // AD変換値を端点時間にセット
    if(!keyDASH)flagDASH = false;   // 長点Keyが離れていれば、フラグクリア
}

void delayDASH(int len){
    while(len--){                   // 指定の長さだけ繰り返す
        __delay_ms(1);              // 1ms 待ち
        if(keyDOT) flagDOT = true;  // 短点が押されたなら短点記憶
    }
}

// サイドトーンの周波数を変更する
void changeFreq(void){
    switch(T2CON){
      case 0x84: T2CON = 0x85; break; // 666Hz
      case 0x85: T2CON = 0x83; break; // 1 kHz
      default:   T2CON = 0x84; break; // 800Hz
    }
    LATA5 = 1;         // LED を点灯し音を出す
    while(!RC4);        // SW が離れるのを待つ
    LATA5 = 0;         // LED を消灯
}

// 割り込み処理関数設定 ---------------------------
void MyTMR_ISR(void){
    PWM3DCH = wave[p_wave];         // 波形定数を更新
    if(p_wave >= 19){               // 零点なら
        if(CWout) p_wave = 0;       // CWout ON で継続
    }else{                          // 零点以外なら
        p_wave ++;                  // 　波形定数を進める
    }
}

/** End of File */